#ifndef ATOMIC_BLOCK_H_
#define ATOMIC_BLOCK_H_

#include <iostream>
#include <vector>
#include <unordered_map>
#include <string>
#include <algorithm>
#include <utility>
#include "cut.hpp"

class Atomic
{
    public:
        Atomic (std::vector<uint32_t> inputs, std::vector<uint32_t> outputs, std::vector<uint32_t> nodeIndexList, std::string truthTable, std::vector<std::vector<uint32_t>> fanoutsVec );
        Atomic (std::vector<Cutt> cuts);

        std::vector<uint32_t> returnInputs() const;
        std::vector<uint32_t> returnOutputs() const;
        std::vector<uint32_t> returnNodeIndexes() const;
        std::string returnTruthTable() const;
        
        friend std::ostream &operator<<( std::ostream &output, const Atomic &A );

        //identifying HAs and FAs and returning them as a pair
        static std::pair<std::vector<Atomic>, std::vector<Atomic>> identifyHAsFAs(std::vector<Cutt*> cuts);

        //Removing HAs inside FAs
        static void removeHAsInFAs(std::pair<std::vector<Atomic>, std::vector<Atomic>> &atomicsPair);

        //Removing atomic blocks with intersection particulary HAs with shared nodes
        static void removeAtomicsWithIntersection(std::vector<Atomic> &atomics);

    private:
        //inputs of the atomic block
        std::vector<uint32_t> inputs;

        //outputs of the atomic block
        std::vector<uint32_t> outputs;

        //the complete list of nodes' indexes in the atomic block
        std::vector<uint32_t> nodeIndexList;

        //atomic block truth table
        std::string truthTable;

        //Fanouts for each atomic block output
        std::vector<std::vector<uint32_t>> fanoutsVec;
};

#endif