#ifndef CUT_H_
#define CUT_H_

#include <vector>
#include <iostream>
#include <string>
#include <algorithm>
#include <mockturtle/mockturtle.hpp>
#include <lorina/aiger.hpp>
#include "cut_type.hpp"

class Cutt
{
    friend class Atomic;

    public:
        Cutt (std::vector<uint32_t> inputs, uint32_t output, std::vector<uint32_t> nodeIndexList, std::string truthTable, std::vector<uint32_t> fanouts);
        Cutt (mockturtle::cut<16, mockturtle::cut_data<true, mockturtle::empty_cut_data> > cut, mockturtle::node<mockturtle::aig_network> rootNode, std::string truthTable, const mockturtle::aig_network& aig, mockturtle::fanout_view<mockturtle::aig_network> view_fanout);
        //~Cutt();

        std::vector<uint32_t> returnInputs() const;
        uint32_t returnOutput() const;
        std::string returnTruthTable() const;
        std::string returnInputSig() const;
        std::vector<uint32_t> returnNodeIndexes() const;

        friend std::ostream &operator<<( std::ostream &output, const Cutt &C );

        //Identifying a cut based on its truth table
        CutType identifyCut () const;

        //Extract cuts for a given AIG
        //cut_size: maximum number of leaves
        //cut_limit: maximum number of extracted cuts for a root node
        static std::vector<Cutt*> ExtractCuts(const mockturtle::aig_network &aig, uint32_t cut_size, uint32_t cut_limit);



    private:
        //inputs of the cut
        std::vector<uint32_t> inputs; 

        //output of the cut
        uint32_t output;

        //the complete list of nodes' indexes in the cut
        std::vector<uint32_t> nodeIndexList;

        //Cut truth table (the values are hexadecimal)
        std::string truthTable; 

        //the input signature of the cut (we create it using the cut inputs)
        std::string inputSig;

        //Fanouts of the cut
        std::vector<uint32_t> fanouts;

};


#endif